@@ -169,11 +169,15 @@ class Conf:
                     'libdir',
                     'libexecdir',
                     'bindir',
+                    'sbindir',
                     'includedir',
                     'datadir',
                     'mandir',
+                    'infodir',
                     'localedir',
                     'sysconfdir',
+                    'localstatedir',
+                    'sharedstatedir',
                 ]:
             parr.append([key, coredata.get_builtin_option_description(key),
                 self.coredata.get_builtin_option(key), coredata.get_builtin_option_choices(key)])
