@@ -49,13 +49,13 @@ class PkgConfigModule:
             # 'os.path.join' for details)
             ofile.write('libdir=%s\n' % os.path.join('${prefix}', coredata.get_builtin_option('libdir')))
             ofile.write('includedir=%s\n' %  os.path.join('${prefix}', coredata.get_builtin_option('includedir')))
+            ofile.write('\n')
             ofile.write('Name: %s\n' % name)
             if len(description) > 0:
                 ofile.write('Description: %s\n' % description)
             if len(url) > 0:
                 ofile.write('URL: %s\n' % url)
-            if len(version) > 0:
-                ofile.write('Version: %s\n' % version)
+            ofile.write('Version: %s\n' % version)
             if len(pub_reqs) > 0:
                 ofile.write('Requires: {}\n'.format(' '.join(pub_reqs)))
             if len(priv_reqs) > 0:
@@ -111,9 +111,9 @@ class PkgConfigModule:
         libs = self.process_libs(kwargs.get('libraries', []))
         priv_libs = self.process_libs(kwargs.get('libraries_private', []))
         subdirs = mesonlib.stringlistify(kwargs.get('subdirs', ['.']))
-        version = kwargs.get('version', '')
+        version = kwargs.get('version', None)
         if not isinstance(version, str):
-            raise mesonlib.MesonException('Version must be a string.')
+            raise mesonlib.MesonException('Version must be specified.')
         name = kwargs.get('name', None)
         if not isinstance(name, str):
             raise mesonlib.MesonException('Name not specified.')
@@ -138,7 +138,7 @@ class PkgConfigModule:
         self.generate_pkgconfig_file(state, libs, subdirs, name, description, url,
                                      version, pcfile, pub_reqs, priv_reqs,
                                      conflicts, priv_libs)
-        return build.Data(False, state.environment.get_scratch_dir(), [pcfile], pkgroot)
+        return build.Data(mesonlib.File(True, state.environment.get_scratch_dir(), pcfile), pkgroot)
 
 def initialize():
     return PkgConfigModule()
